package me.icymint.libra.sage.model.ddl;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import me.icymint.libra.sage.core.dialect.SqlDialect;
import me.icymint.libra.sage.core.result.AbstractResultHandler;
import me.icymint.libra.sage.core.result.MetaDataResultHandler;
import me.icymint.libra.sage.model.SqlFactory;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlType;
import me.icymint.libra.sage.model.SqlVarType;

/**
 * 从数据库中自动抓取表格。
 * 
 * @author Daniel
 * @datetime 2012-12-18下午4:33:30
 * 
 */
public class FetchTable extends AbstractResultHandler<SqlTable> implements
		MetaDataResultHandler<SqlTable> {

	private final SqlFactory sf;
	private final String table;
	private List<String> keys;
	private final String new_table;
	private Map<String, Object[]> fkeys = new LinkedHashMap<String, Object[]>();
	private String[] uniques;
	private boolean mask = true;
	private Connection conn = null;

	public FetchTable(SqlFactory sf, String table) {
		this(sf, table, null);
	}

	public FetchTable(SqlFactory sf, String table, String new_table) {
		super(1);
		this.sf = sf;
		this.table = table.toLowerCase();
		if (new_table == null)
			new_table = table;
		this.new_table = new_table.toLowerCase();
	}

	private String getPureTableName(String name) {
		String x = sf.getDatabase().getSqlInfo().getPrefix();
		if (!x.equals("") && name.startsWith(x)) {
			return name.substring(x.length() + 1);
		} else
			return name;
	}

	private String getTableName() {
		String x = sf.getDatabase().getSqlInfo().getPrefix();
		return x.equals("") ? table : x + "_" + table;
	}

	private String getTableName(DatabaseMetaData metadata) throws SQLException {
		if (!mask || sf.getDatabase().getPrefix().equals("")) {
			return table;
		} else {
			ResultSet rs = metadata.getTables(sf.getDatabase().getName(), null,
					table, new String[] { "TABLE" });
			while (rs.next()) {
				mask = false;
			}
			rs.close();
			return mask ? this.getTableName() : table;
		}
	}

	@Override
	public SqlTable handle(ResultSet rs) throws SQLException {
		SqlDialect sd = sf.getDatabase().getDialect();
		SqlTable t = sf.createTable(null, new_table, mask);
		while (rs.next()) {
			SqlVarType tp = sd.transfer(rs.getInt("DATA_TYPE"));
			SqlType type;
			if (tp.hasLength())
				type = sf.createType(tp, rs.getInt("COLUMN_SIZE"));
			else {
				type = sf.createType(tp);
			}
			String name = rs.getString("COLUMN_NAME").toLowerCase();
			String def = rs.getString("COLUMN_DEF");
			if (def != null) {
				if (!def.contains(" ") && def.matches("\\('.*'\\)")) {
					def = def.substring(2, def.length() - 2);
				} else if (!def.matches("\\d+(.\\d+)?"))
					def = null;
			}
			t.addVar(sf.createVar(name, type, def));
			if (rs.getInt("NULLABLE") == 0)
				t.setNotNull(name);
		}
		setAutoIncrement(t);
		t.setPrimaryKeys(keys.toArray(new String[] {}));
		if (t.getVarSize() == 0) {
			sf.getDatabase().delTable(t);
			throw new SQLException("数据表" + this.getTableName() + "抓取失败！");
		}
		for (String var : fkeys.keySet()) {
			String f = (String) fkeys.get(var)[0];
			SqlTable ft = (SqlTable) fkeys.get(var)[1];
			t.setForeignKey(var, ft, f);
		}
		for (String var : uniques) {
			t.setUniqueVar(var);
		}
		return t;
	}

	/**
	 * 获取外键。
	 * 
	 * @param rs
	 * @throws SQLException
	 */
	protected Map<String, Object[]> handleFKeys(ResultSet rs)
			throws SQLException {
		while (rs.next()) {
			String ftn = getPureTableName(rs.getString("PKTABLE_NAME")
					.toLowerCase());
			SqlTable ft = null;
			try {
				ft = sf.getDatabase().getTable(ftn);
			} catch (SQLException e) {
				if (!ftn.equals(table))
					ft = sf.fetchTable(ftn);
			}
			if (ft != null)
				fkeys.put(
						rs.getString("FKCOLUMN_NAME").toLowerCase(),
						new Object[] {
								rs.getString("PKCOLUMN_NAME").toLowerCase(), ft });
			else {
				// System.out.println(ftn+"--"+rs.getString("FKCOLUMN_NAME").toLowerCase()
				// +"---"+ rs.getString("PKCOLUMN_NAME"));
			}
		}
		rs.close();
		return fkeys;
	}

	protected void handleIndex(ResultSet rs) throws SQLException {
		while (rs.next()) {
			System.out.println(rs.getString("ORDINAL_POSITION") + "--"
					+ rs.getString("NON_UNIQUE") + "--"
					+ rs.getString("TABLE_NAME") + "--"
					+ rs.getString("COLUMN_NAME"));
		}
		rs.close();
	}

	/**
	 * 获取主键
	 * 
	 * @param rs
	 * @throws SQLException
	 */
	protected List<String> handleKeys(ResultSet rs) throws SQLException {
		List<String> list = new LinkedList<String>();
		while (rs.next()) {
			list.add(rs.getString("COLUMN_NAME").toLowerCase());
		}
		rs.close();
		return list;
	}

	protected String[] handleUnique(ResultSet rs) throws SQLException {
		List<String> list = new LinkedList<String>();
		while (rs.next()) {
			String u = rs.getString("COLUMN_NAME");
			if (u != null && !keys.contains(u.toLowerCase())) {
				list.add(u);
			}
		}
		rs.close();
		return list.toArray(new String[] {});
	}

	@Override
	public ResultSet queryResultSet(DatabaseMetaData metadata)
			throws SQLException {
		String name = getTableName(metadata);
		// handleIndex(metadata.getIndexInfo(sf.getDatabase().getName(), null,
		// name, false, false));
		keys = handleKeys(metadata.getPrimaryKeys(sf.getDatabase().getName(),
				null, name));
		fkeys = handleFKeys(metadata.getImportedKeys(
				sf.getDatabase().getName(), null, name));
		uniques = handleUnique(metadata.getIndexInfo(
				sf.getDatabase().getName(), null, name, true, false));
		conn = metadata.getConnection();
		return metadata
				.getColumns(sf.getDatabase().getName(), null, name, null);
	}

	private void setAutoIncrement(SqlTable t) throws SQLException {
		if (conn != null) {
			PreparedStatement pps = conn.prepareStatement(t.getDialect()
					.getQueryAll(t));
			pps.setMaxRows(1);
			ResultSet rrs = pps.executeQuery();
			ResultSetMetaData md = rrs.getMetaData();
			boolean flag = false;
			for (int i = 1; i <= md.getColumnCount(); i++) {
				String var = md.getColumnName(i);
				if (!flag && md.isAutoIncrement(i)) {
					flag = true;
					t.setAutoIncrement(true);
					t.setPrimaryKeys(var);
				}
				String label = md.getColumnLabel(i);
				if (label != null && !label.equals("") && !var.equals(label)) {
					t.setLabel(label);
				}
			}
			rrs.close();
			pps.close();
		}
	}

}
